/*
 *  DaysOffSchoolWindow.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Reflection;
using Gtk;
using Glade;
using gnutu.Widgets;
using Mono.Unix;


namespace gnutu.GUI {

	/*
	 *  DaysOffSchoolWindow class
	 */
	
	public class DaysOffSchoolWindow {
		Glade.XML gui;
		
		[Glade.Widget] public Gtk.Window   Window;
		[Glade.Widget] public Gtk.TreeView DaysList;
		
		
		public DaysOffSchoolWindow ()
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "DaysOffSchoolWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("DaysOffSchoolWindow");
			
			
			ListStore store = new ListStore (typeof (string), typeof (string),  typeof (string), typeof (string));
			store.SetSortColumnId (0, Gtk.SortType.Ascending);
			DaysList.Model = store;
			
			DaysList.AppendColumn (Catalog.GetString ("Date"), new CellRendererText (), "text", 0);
			DaysList.AppendColumn (Catalog.GetString ("Description"), new CellRendererText (), "text", 1);
			
			store.SetSortFunc (0, new Gtk.TreeIterCompareFunc (DaysListCompare));
			
			
			foreach (DayOffSchool day in DaysOffSchool.data)
				store.AppendValues (new object [] {day.date.ToString (Settings.GetBool ("DateFirstYear")), day.description});
		}
		
		
		public void on_SettingsWindow_delete_event (object o, DeleteEventArgs args) 
		{
			args.RetVal = false;
		}
		
		
		public void on_OkBtn_clicked (object o, EventArgs args)
		{
			DaysOffSchool.Clear ();
			
			Gtk.ListStore store = (Gtk.ListStore) DaysList.Model;
			Gtk.TreeIter  itr;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr)) {
				DayOffSchool day = new DayOffSchool ();
				day.date         = new Date ((string) store.GetValue (itr, 0), Settings.GetBool ("DateFirstYear"));
				day.description  = (string) store.GetValue (itr, 1);
				
				DaysOffSchool.Add (day);
			}
			
			
			Window.Destroy ();
		}
		
		
		public void on_CancelBtn_clicked (object o, EventArgs args)
		{
			Window.Destroy ();
		}
		
		
		public void on_AddBtn_clicked (object o, EventArgs args)
		{
			ModifyDayOffSchoolWindow wdw = new ModifyDayOffSchoolWindow ((Gtk.ListStore) DaysList.Model);
			wdw.Window.Show ();
			
			Gtk.Button RemoveBtn = (Gtk.Button) gui.GetWidget ("RemoveBtn");
			RemoveBtn.Sensitive  = false;
		}
		
		
		public void on_RemoveBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (DaysList.Selection.GetSelected (out mod, out itr)) {
				Gtk.ListStore store = (Gtk.ListStore) DaysList.Model;
				store.Remove (ref itr);

				Gtk.Button btn = (Gtk.Button) o;
				btn.Sensitive = false;
			}
		}
		
		
		public void on_ModifyBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (DaysList.Selection.GetSelected (out mod, out itr)) {
				ModifyDayOffSchoolWindow wdw = new ModifyDayOffSchoolWindow ((Gtk.ListStore) DaysList.Model, itr);
				wdw.Window.Show ();
			}
		}
		
		
		public void on_DaysList_cursor_changed (object o, EventArgs args)
		{
			Gtk.Button RemoveBtn = (Gtk.Button) gui.GetWidget ("RemoveBtn");
			Gtk.Button ModifyBtn = (Gtk.Button) gui.GetWidget ("ModifyBtn");
			
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (DaysList.Selection.GetSelected (out mod, out itr)) {
				RemoveBtn.Sensitive = true;
				ModifyBtn.Sensitive = true;
			} else {
				RemoveBtn.Sensitive = false;
				ModifyBtn.Sensitive = false;
			}
		}
		
		
		private int DaysListCompare (Gtk.TreeModel model, Gtk.TreeIter a, Gtk.TreeIter b)
		{
			Date t1 = new Date ((string) model.GetValue (a, 0), Settings.GetBool ("DateFirstYear"));
			Date t2 = new Date ((string) model.GetValue (b, 0), Settings.GetBool ("DateFirstYear"));
			
			if (t1.ToDateTime () > t2.ToDateTime ())
				return 1;
			else if (t1.ToDateTime () == t2.ToDateTime ())
				return 0;
			
			return -1;
		}
	}
	
	
	
	/*
	 *  ModifyDayOffSchoolWindow class
	 */
	
	public class ModifyDayOffSchoolWindow {
		private Glade.XML gui;
		
		[Glade.Widget] public  Gtk.Window      Window;
		[Glade.Widget] private Gtk.Entry       Description;
		[Glade.Widget] private Gtk.Label       FromLabel;
		[Glade.Widget] private Gtk.Label       ToLabel;
		[Glade.Widget] private Gtk.HBox        FromHBox;
		[Glade.Widget] private Gtk.HBox        ToHBox;
		[Glade.Widget] private Gtk.Frame       DateFrame;
		[Glade.Widget] private Gtk.CheckButton Overwrite;
		[Glade.Widget] private Gtk.CheckButton MoreDays;
		
		private DateEdit begin;
		private DateEdit end;
		
		private bool          modify;
		private Gtk.TreeIter  iter;
		private Gtk.ListStore store;
		
		
		public ModifyDayOffSchoolWindow (Gtk.ListStore store)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "ModifyDayOffSchoolWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("ModifyDayOffSchoolWindow");
			
			
			this.begin = new DateEdit ();
			this.end   = new DateEdit ();
			FromHBox.Add (this.begin);
			ToHBox.Add   (this.end);
			
			FromLabel.Text    = Catalog.GetString ("Date:");
			ToLabel.Visible   = false;
			ToHBox.Visible    = false;
			DateFrame.Visible = true;
			Overwrite.Visible = false;
			
			
			this.modify = false;
			this.store  = store;
		}
		
		
		public ModifyDayOffSchoolWindow (Gtk.ListStore store, Gtk.TreeIter iter)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "ModifyDayOffSchoolWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("ModifyDayOffSchoolWindow");
			
			DateFrame.Visible = false;
			
			this.modify = true;
			this.iter   = iter;
			this.store  = store;
			
			this.Description.Text = (string) this.store.GetValue (iter, 1);
		}
		
		
		public void on_Description_activate (object o, EventArgs args)
		{
			on_OkBtn_clicked (null, null);
		}
		
		
		public void on_OkBtn_clicked (object o, EventArgs args)
		{
			if (modify == true)
				store.SetValue (iter, 1, Description.Text);
			else {
				if (MoreDays.Active == false) {
					DateTime     date = begin.GetDate ();
					
					if (DayExists (date)) {
						Gtk.MessageDialog dialog = new Gtk.MessageDialog (Window,
													DialogFlags.DestroyWithParent,
													MessageType.Warning,
													ButtonsType.Close,
													Catalog.GetString ("Day with this date exists !\nEnter other date."));
						
						dialog.Run ();
						dialog.Destroy ();
							
						return;
					}
					
					store.AppendValues (new object [] {begin.GetDate2 ().ToString (Settings.GetBool ("DateFirstYear")), Description.Text});
				} else {
					DateTime date1 = begin.GetDate ();
					DateTime date2 = end.GetDate ();
					
					while (date1 <= date2) {
						if (DayExists (date1)) {
							if (Overwrite.Active == true)
								store.SetValue (GetDay (date1), 1, Description.Text);
						} else
							store.AppendValues (new object [] {new Date (date1).ToString (Settings.GetBool ("DateFirstYear")), Description.Text});
						
						date1 = date1.AddDays (1);
					}
				}
			}
			
			Window.Destroy ();
		}
		
		
		public void on_CancelBtn_clicked (object o, EventArgs args)
		{
			Window.Destroy ();
		}
		
		
		public void on_MoreDays_toggled (object o, EventArgs args)
		{
			Gtk.ToggleButton btn = (Gtk.ToggleButton) o;
			
			if (btn.Active == true) {
				FromLabel.Text    = Catalog.GetString ("From:");
				ToLabel.Visible   = true;
				ToHBox.Visible    = true;
				Overwrite.Visible = true;
			} else {
				FromLabel.Text    = Catalog.GetString ("Date:");
				ToLabel.Visible   = false;
				ToHBox.Visible    = false;
				Overwrite.Visible = false;
			}
		}
		
		
		private bool DayExists (DateTime date)
		{
			Gtk.TreeIter itr;
			string       date2 = new Date (date).ToString (Settings.GetBool ("DateFirstYear"));
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr)) {
				if (date2 == (string) store.GetValue (itr, 0))
					return true;
			}
			
			return false;
		}
		
		
		private Gtk.TreeIter GetDay (DateTime date)
		{
			Gtk.TreeIter itr;
			string       date2 = new Date (date).ToString (Settings.GetBool ("DateFirstYear"));
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr)) {
				if (date2 == (string) store.GetValue (itr, 0))
					return itr;
			}
			
			return new Gtk.TreeIter ();
		}
	}
}
