/*
 *  DateEdit.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using GLib;
using gnutu;


namespace gnutu.Widgets {
	public class DateEdit : Gtk.HBox {
		private Gtk.Entry        DateEntry;
		private Gtk.ToggleButton SelectBtn;
		private Gtk.Window       CalendarWindow;
		private Gtk.Calendar     Calendar;
		
		
		public DateEdit ()
		{
			// This
			this.Spacing = 1;
			
			
			// CalendarWindow and Calendar widgets
			CalendarWindow = new Gtk.Window (" ");
			CalendarWindow.TypeHint = Gdk.WindowTypeHint.Normal;
			CalendarWindow.Decorated = false;
			CalendarWindow.WindowPosition = Gtk.WindowPosition.Mouse;
			CalendarWindow.Modal = true;
			CalendarWindow.FocusOutEvent += new Gtk.FocusOutEventHandler
												(on_CalendarWindow_focus_out);
			
			this.Calendar = new Gtk.Calendar ();
			this.Calendar.Show ();
			this.Calendar.Date = System.DateTime.Today;
			this.Calendar.DaySelected += new EventHandler (on_Calendar_day_selected);
			this.Calendar.DaySelectedDoubleClick += new EventHandler
									(on_Calendar_day_selected_double_clicked);
			CalendarWindow.Add (this.Calendar);
			
			// DateEntry
			DateEntry = new Gtk.Entry ();
			DateEntry.WidthRequest = 100;
			DateEntry.IsEditable = false;
			DateEntry.Text = FormatDate (this.Calendar.Date);
			DateEntry.Show ();
			this.PackStart (DateEntry, true, true, 1);
			
			
			// SelectBtn
			SelectBtn = new Gtk.ToggleButton ();
			SelectBtn.Add (new Gtk.Image (Gtk.Stock.GoDown, Gtk.IconSize.Menu));
			SelectBtn.Clicked += new EventHandler (on_SelectBtn_clicked);
			this.PackEnd (SelectBtn, false, true, 0);
			
			
			this.ShowAll ();
		}
		
		
		/* EVENTS *************************************************************/
		private void on_SelectBtn_clicked (object obj, EventArgs args)
		{
			Gtk.ToggleButton btn = (Gtk.ToggleButton) obj;
			
			if (btn.Active == true) {
				CalendarWindow.Show ();
			} else {
				CalendarWindow.Hide ();
			}
		}
		
		
		private void on_CalendarWindow_focus_out (object obj, Gtk.FocusOutEventArgs args)
		{
			CalendarWindow.Hide ();
			SelectBtn.Active = false;
		}
		
		
		private void on_Calendar_day_selected (object o, EventArgs args)
		{
			DateEntry.Text = FormatDate (this.Calendar.Date);
		}
		
		
		private void on_Calendar_day_selected_double_clicked (object o, EventArgs args)
		{
			CalendarWindow.Hide ();
			SelectBtn.Active = false;
		}
		
		
		/* PRIVATE FUNCTIONS **************************************************/
		public string FormatDate (System.DateTime date)
		{
			int    MonthInt = this.Calendar.Month + 1;
			string Date, Day, Month;
			
			
			if (MonthInt <= 9)
				Month = "0" + MonthInt.ToString ();
			else
				Month = MonthInt.ToString ();
			
			if (this.Calendar.Day <= 9)
				Day = "0" + this.Calendar.Day.ToString ();
			else
				Day = this.Calendar.Day.ToString ();
			
			
			if (gnutu.Settings.GetBool ("DateFirstYear") == true)
				Date = this.Calendar.Year.ToString () + "-" + Month + "-" + Day;
			else
				Date = Day + "-" + Month + "-" + this.Calendar.Year.ToString ();
			
			
			return Date;
		}
		
		
		/* PUBLIC FUNCTIONS ***************************************************/
		public void SetDate (System.DateTime date)
		{
			this.Calendar.Date = date;
		}
		
		
		public string GetDateText ()
		{
			return DateEntry.Text;
		}
		
		
		public DateTime GetDate ()
		{
			return this.Calendar.Date;
		}
		
		
		public Date GetDate2 ()
		{
			return new Date (this.Calendar.Date);
		}
	}
}
